#! /bin/sh
# This is a shell archive.  Remove anything before this line, then unpack
# it by saving it into a file and typing "sh file".  To overwrite existing
# files, type "sh file -c".  You can also feed this as standard input via
# unshar, or by typing "sh <file", e.g..  If this archive is complete, you
# will see the following message at the end:
#		"End of archive 2 (of 2)."
# Contents:  methods/Examples/dataflow/dataFlow.scm
#   methods/Examples/matrices/Matrices.scm
#   methods/Examples/utils/kstream.scm
# Wrapped by oz@yunexus on Wed May 22 12:03:17 1991
PATH=/bin:/usr/bin:/usr/ucb ; export PATH
if test -f 'methods/Examples/dataflow/dataFlow.scm' -a "${1}" != "-c" ; then 
  echo shar: Will not clobber existing file \"'methods/Examples/dataflow/dataFlow.scm'\"
else
echo shar: Extracting \"'methods/Examples/dataflow/dataFlow.scm'\" \(19701 characters\)
sed "s/^X//" >'methods/Examples/dataflow/dataFlow.scm' <<'END_OF_FILE'
X; dataFlow.scm; simple data driven data flow executive.  
X
X;================================================================
X;| Brian Beckman                  | brian@topaz.jpl.nasa.gov    |
X;| Computer Graphics Laboratory   | (818) 397-9207              |
X;| Mail Stop 510-202              | (818) 397-9344              |
X;| Jet Propulsion Laboratory      |                             |
X;| Pasadena, CA 91109             | 6 July 1989                 |
X;================================================================
X
X(load "methods.scm")
X(load "queues.scm")
X
X; This file contains a simple data-driven data flow executive, some 
X; basic primitive data flow node classes, and an extended example.
X; The example is a data flow program for a quadratic equation solver
X; for quadratic equations with real roots.
X;
X; The procedures in this file make up a small data flow language. This
X; language has the following structure.  A data flow ``graph'' is a
X; list of ``nodes''. A ``node'' is either a ``primitive node'' or a
X; graph.  Thus, a graph may contain embedded or nested sub-graphs.
X; Every node has zero or more ``input arcs'' and zero or more ``output
X; arcs''.  A node operates by consuming tokens from its input arcs,
X; performing an operation on the tokens, and appending tokens to its
X; output arcs.  Primitive arcs differ from graphs in how the operation
X; is implemented.  The operation of a primitive node is represented by
X; a procedure that takes K arguments and produces a LIST of L output
X; values, where K is the number of input arcs and L is the number of
X; output arcs of the node.  The operation of a graph is represented by
X; a number of internal nodes and arcs that together perform a
X; composite action.  The input arcs and output arcs of a graph are
X; those with only one end connected to a node belonging to the graph.
X; 
X; Arcs are FIFO queues.  They may be represented graphically as arrows.
X; The node at the tail of an arc appends tokens to the tail of the 
X; queue, and the node at the head of an arc consumes tokens from the
X; head of the queue.  A node is ``ready to fire'' when all of its
X; input arcs are non-empty.  When a node fires, exactly one token is
X; taken from each input arc and exactly one token is pushed into each
X; output arc.
X; 
X; Some nodes types have no input arcs.  They are ``data sources'', and
X; they are always ready to fire, by convention.  Other node types have
X; no output arcs.  They are called ``data sinks''. 
X; 
X; Every node has methods for the following messages.
X; 'ready?  -- return #t if the node is ready to fire.
X; 'fire!   -- fire the node, modifying the input and output arcs
X; 'print   -- print a list of the front tokens in each input arc
X; 'clear!  -- clear all input and output arcs.
X;
X; This file contains the following procedures, most of which 
X; represent node classes:
X;   new-pnode . . . . . . . create a primitive node
X;   new-dfgraph . . . . . . create a data flow graph
X;   execute-data-flow . . . execute a data flow graph until done
X;   new-add-node. . . . . . create an adder
X;   new-mul-node. . . . . . create a multiplier
X;   new-div-node. . . . . . create a divider
X;   new-negate-node . . . . create a unary minus operator
X;   new-sub-node. . . . . . create a subtracter
X;   new-dup-node. . . . . . create a duplicator
X;   new-sqr-node. . . . . . create a squaring node
X;   new-const-node. . . . . create a constant producer
X;   new-sqrt-node . . . . . create a square root-taking node
X;   new-disc-node . . . . . create a discriminant calculator
X;   new-dbl-node. . . . . . create a doubler
X;   new-quadratic-solver. . create a real root finder for quadratics
X;
X; 
X; Create a primitive node.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                      N E W - P N O D E                     ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-pnode proc input-queues output-queues)
X  
X  (let* (  (supers ())  ) ;;; a root class
X    
X    (define (all-qs-non-empty? qlist)
X      (cond
X       (  (null? qlist)  #t  ) 
X       ;;; if there are no queues, then all queues are non-empty, 
X       ;;; by convention
X       (  else  (and (not (send (car qlist) 'empty?))
X                     (all-qs-non-empty? (cdr qlist)))  )))
X    
X    (define (ready-to-fire?)
X      (all-qs-non-empty? input-queues))
X    
X    (define (fire-node!)
X      (let* (  (input-list 
X                (map (lambda (q) (send q 'front!)) input-queues))
X               (output-list (apply proc input-list))  )
X        
X        (define (iter output-items output-qs)
X          (cond
X           (  (null? output-qs)     self  )  ;;; return self for method
X           (  (null? output-items)  self  )  ;;; composition purposes
X           (  else  (let (  (o (car output-items))
X                            (q (car output-qs))  )
X                      (send q 'append! o)
X                      (iter (cdr output-items) (cdr output-qs)))  )))
X        
X        (iter output-list output-queues)))
X
X    (define (print)
X      (define (front q)
X        (if (send q 'empty?) "Empty" (send q 'front)))
X      (display
X       (map front input-queues))
X      (newline)
X      self)
X    
X    (define (clear!)
X      (map (lambda (q) (send q 'clear!)) input-queues)
X      (map (lambda (q) (send q 'clear!)) output-queues)
X      self)
X    
X    (define (self msg)
X      (cond
X       (  (eq? msg 'ready?)  ready-to-fire?  )
X       (  (eq? msg 'fire!)   fire-node!  )
X       (  (eq? msg 'print)   print  )
X       (  (eq? msg 'clear!)  clear!  )
X       (  (search-supertypes supers msg)  )
X       (  else  (make-error-method "Data-flow-node" msg)  )))
X    
X    self))
X
X; Create a data flow graph given a list of nodes, a list of 
X; input arcs, and a list of output arcs.  These arcs are the
X; ones connected at exactly one end to nodes in the graph.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - D F G R A P H                   ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-dfgraph graph input-queues output-queues)
X  
X  (let* (  (supers ())  )
X    
X    (define (all-qs-non-empty? qlist)
X      (cond
X       (  (null? qlist)  #t  )
X       (  else  (and (not (send (car qlist) 'empty?))
X                     (all-qs-non-empty? (cdr qlist)))  )))
X    
X    (define (ready-to-fire?)
X      (all-qs-non-empty? input-queues))
X    
X    (define (print graph)
X      (cond
X       (  (null? graph)  self  )
X       (  else  (send (car graph) 'print)
X                (print (cdr graph)))))
X    
X    (define (clear! graph)
X      (cond
X       (  (null? graph)  self  )
X       (  else  (send (car graph) 'clear!)
X                (clear! (cdr graph)))))
X    
X    (define (self msg)
X      (cond
X       (  (eq? msg 'ready?)  ready-to-fire?  )
X       (  (eq? msg 'fire!)   (lambda () (execute-data-flow graph))  )
X       (  (eq? msg 'print)   (lambda () (print graph))  )
X       (  (eq? msg 'clear!)  (lambda () (clear! graph)) )
X       (  (search-supertypes supers msg)  )
X       (  else  (make-error-method "DFGraph" msg)  )))
X    
X    self))
X
X; This is the data flow executve.  It takes a
X; graph as input, which is a list of nodes.  It loops over the 
X; list until nothing more is ready.  The nodes, which are
X; connected by shared arcs, must form a directed acyclic graph.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;              E X E C U T E - D A T A - F L O W             ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (execute-data-flow graph)
X  
X  (define (one-pass graph)
X    (cond
X     (  (null? graph)  #f  ) ;;; so far, nothing is ready.
X     
X     (  (send (car graph) 'ready?)  (send (car graph) 'fire!)
X                                    (or #t (one-pass (cdr graph)))  )
X     
X        ;;; at least one node was ready, so we OR in #t.
X     
X     (  else (one-pass (cdr graph))  )))
X  
X  (define (iter) ;;; loop until nothing is ready.
X    (cond
X     (  (not (one-pass graph))  'done  )
X     (  else  (iter)  )))
X  
X  (iter))
X
X; The first primitive node is the ``add-node''.  An add-node
X; adds the tokens in its input arcs.  It has two input arcs and one
X; output arc.  Since the add operation is inherently commutative,
X; the behavior of an add-node is insensitive to the order of its
X; input arcs.  That is, data coming down the left-input is 
X; treated the same as data coming down the right-input by this
X; type of node.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                   N E W - A D D - N O D E                  ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-add-node left-input right-input output)
X  
X    (let* ((supers (list
X                      (new-pnode
X                       (lambda (a b) (list (+ a b)))
X                       (list left-input right-input)
X                       (list output)))))
X
X    (define (self msg)
X      (cond
X       (  (search-supertypes supers msg)  )
X       (  else  (make-error-method "Add-node" msg)  )))
X    
X    self))
X
X;;; Test suite for add-nodes and execute-data-flow 
X
X(define arc1 (new-queue '(10 20 30 40)))
X
X(define arc2 (new-queue '(5 5 5)))
X
X(define o-arc (new-queue))
X
X(define an-add-node (new-add-node arc1 arc2 o-arc))
X
X(define (print-all)
X  (send arc1 'print)
X  (send arc2 'print)
X  (send o-arc 'print))
X
X(print-all)
X;(10 20 30 40)
X;(5 5 5)
X;()
X
X(execute-data-flow (list an-add-node))
X;'done
X
X(print-all)
X;(40)
X;()
X;(15 25 35)
X; 
X; Data has flowed out of the input queues and into the 
X; output queue.  One input token is left behind because it had no
X; positionally corresponding token in the other input queue.
X
X;;; End of test suite
X
X; The next pnode is a multiplier, another commutative operator.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - M U L - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-mul-node left-input right-input output)
X  
X  (let* ((supers (list
X                  (new-pnode
X                   (lambda (a b) (list (* a b)))
X                   (list left-input right-input)
X                   (list output)))))
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Mul-node" msg)  )))
X    
X    self))
X
X; The next pnode is a divider, where the dividend comes in the left
X; arc and the divisor comes in the right arc.  
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - D I V - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-div-node left-input right-input output)
X  
X  (let* ((supers (list
X                  (new-pnode
X                   (lambda (a b) (list (/ a b)))
X                   (list left-input right-input)
X                   (list output)))))
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Div-node" msg)  )))
X    
X    self))
X
X; The next pnode is unary minus operator.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                N E W - N E G A T E - N O D E               ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-negate-node input output)
X  
X  (let* ((negate-proc (lambda (in) (list (- in))))
X         (supers (list
X                  (new-pnode
X                   negate-proc
X                   (list input)
X                   (list output)))))
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Negate-node" msg)  )))
X    
X    self))
X
X; The subtraction node class is defined as a dfgraph
X; composing a negation and an addition.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - S U B - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-sub-node left-input right-input output)   ;;; subtraction
X  
X  (let* (  (AQ (new-queue))
X           (N  (new-negate-node right-input AQ))
X           (P  (new-add-node left-input AQ output))
X           (supers (list (new-dfgraph
X                          (list n p)
X                          (list left-input right-input)
X                          (list output))))  )
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Sub-node" msg)  )))
X    
X    self))
X
X; We need a duplicator node class, for replicating flows.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - D U P - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-dup-node input left-output right-output)
X
X  (let* ((supers (list
X                  (new-pnode
X                   (lambda (in) (list in in))
X                   (list input)
X                   (list left-output right-output)))))
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Dup-node" msg)  )))
X    
X    self))
X
X; The square "node" is a dfgraph comprising a dup and multiply.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - S Q R - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-sqr-node in out)
X  
X  (let* (  (AQ (new-queue))
X           (BQ (new-queue))
X           (D  (new-dup-node in AQ BQ))
X           (M  (new-mul-node AQ BQ out))
X           (supers (list (new-dfgraph
X                          (list D M)
X                          (list in)
X                          (list out))))  )
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Sqr-node" msg)  )))
X    
X    self))
X
X; We need a node to produce a constant value when it has anything 
X; whatever in its input queue.  
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                N E W - C O N S T - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-const-node constant inq outq)
X  
X  (let* (  (supers (list
X                    (new-pnode
X                     (lambda (x) (list constant))
X                     (list inq)
X                     (list outq))))  )
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Const-node" msg)  )))
X    
X    self))
X
X; A sqrt node is needed that returns both branches of the square root
X; function (real values only).  This node will blow up on input of
X; a negative number.  
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                  N E W - S Q R T - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-sqrt-node input out1 out2)
X  
X  (let* (  (supers (list
X                    (new-pnode
X                     (lambda (x) (list (sqrt x) (- (sqrt x))))
X                     (list input)
X                     (list out1 out2))))  )
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Sqrt-node" msg)  )))
X    
X    self))
X
X; Here's the graph for the quadratic equation solution
X;
X;              ----------
X;        +    /  2
X;    -b  -  \/  b  - 4ac
X;  --------------------------
X;           2 a
X;
X; 
X; The graph is somewhat complex, and it is really quite
X; impossible to write this code without drawing the graph first.  
X; It is also impossible to paste a picture of the graph in
X; these comments, so the reader will have to reconstruct the graph
X; from the written description.  It is a little tedious, though
X; not difficult to do so.
X; 
X; First, a pair of helper node classes, ``disc'' to evaluate
X; discriminants, and ``dbl'' to double its input.  These classes
X; will encapsulate the constants 4 and 2.
X; 
X;;;------------------------------'-----------------------------;;;
X;;;                  N E W - D I S C - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-disc-node A B C D)
X  
X  (let* ((Z (new-queue))
X         (Y (new-queue))
X         (D1 (new-dup-node A Z Y))
X         (X (new-queue))
X         (M1 (new-mul-node Y C X))
X         (W (new-queue))
X         (C1 (new-const-node 4 Z W))
X         (V (new-queue))
X         (S1 (new-sqr-node B V))
X         (U (new-queue))
X         (M2 (new-mul-node W X U))
X         (SUB1 (new-sub-node V U D))
X         (supers (list
X                    (new-dfgraph
X                     (list S1  D1  M1  C1  M2  SUB1)
X                     (list A   B   C)
X                     (list D)))))
X    
X    (define (self msg)
X      (cond
X       (  (search-supertypes supers msg)  )
X       (  else  (make-error-method "Disc-node" msg)  )))
X    
X    self))
X
X;;;------------------------------'-----------------------------;;;
X;;;                    N E W - D B L - N O D E                 ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-dbl-node A D)
X  
X  (let* ((Z (new-queue))
X         (Y (new-queue))
X         (D1 (new-dup-node A Y Z))
X         (X (new-queue))
X         (C1 (new-const-node 2 Z X))
X         (M1 (new-mul-node Y X D))
X         (supers (list
X                  (new-dfgraph
X                   (list  D1  C1  M1)
X                   (list  A)
X                   (list  D)))))
X    
X    (define (self msg)
X      (cond
X       (  (search-supertypes supers msg)  )
X       (  else  (make-error-method "Dbl-node" msg)  )))
X    
X    self))
X
X;;;------------------------------'-----------------------------;;;
X;;;          N E W - Q U A D R A T I C - S O L V E R           ;;;
X;;;------------------------------.-----------------------------;;;
X
X(define (new-quadratic-solver A B C rroot1 rroot2)
X  
X  (let* ((Z (new-queue))
X         (Y (new-queue))
X         (D1 (new-dup-node A Z Y))
X         (X (new-queue))
X         (W (new-queue))
X         (D2 (new-dup-node B X W))
X         (V (new-queue))
X         (DBL (new-dbl-node Z V))
X         (U (new-queue))
X         (DISC (new-disc-node Y X C U))
X         (T (new-queue))
X         (N1 (new-negate-node W T))
X         (S (new-queue))
X         (R (new-queue))
X         (D4 (new-dup-node V S R))
X         (Q (new-queue))
X         (P (new-queue))
X         (S1 (new-sqrt-node U Q P))
X         (O (new-queue))
X         (N (new-queue))
X         (D3 (new-dup-node T O N))
X         (M (new-queue))
X         (P1 (new-add-node Q O M))
X         (L (new-queue))
X         (P2 (new-add-node P N L))
X         (/1 (new-div-node M S RROOT1))
X         (/2 (new-div-node L R RROOT2))
X         (supers
X          (list
X           (new-dfgraph
X            (list  D1   D2   N1   DISC
X                   DBL  D3   S1   D4 
X                   P1   P2   /1   /2)
X            (list  A    B    C)
X            (list  RROOT1    RROOT2)))))
X    
X    (define (self msg)
X      (cond
X        (  (search-supertypes supers msg)  )
X        (  else  (make-error-method "Quadratic-solver" msg)  )))
X    
X    self))
X
X; Whew!  Now, lets try it.  
X
X(define a (new-queue '( 8  2  3)))
X(define b (new-queue '(10 -7 -9)))
X(define c (new-queue '( 2  5  4)))
X(define r1 (new-queue))
X(define r2 (new-queue))
X(define (print)
X  (send a 'print)
X  (send b 'print)
X  (send c 'print)
X  (send r1 'print)
X  (send r2 'print))
X
X(define qs (new-quadratic-solver a b c r1 r2))
X
X(print)
X(execute-data-flow (list qs))
X(print)
END_OF_FILE
if test 19701 -ne `wc -c <'methods/Examples/dataflow/dataFlow.scm'`; then
    echo shar: \"'methods/Examples/dataflow/dataFlow.scm'\" unpacked with wrong size!
fi
# end of 'methods/Examples/dataflow/dataFlow.scm'
fi
if test -f 'methods/Examples/matrices/Matrices.scm' -a "${1}" != "-c" ; then 
  echo shar: Will not clobber existing file \"'methods/Examples/matrices/Matrices.scm'\"
else
echo shar: Extracting \"'methods/Examples/matrices/Matrices.scm'\" \(10071 characters\)
sed "s/^X//" >'methods/Examples/matrices/Matrices.scm' <<'END_OF_FILE'
X;************************************************************
X;                                                           *
X; Copyright (c) 1990, California Institute of Technology.   *
X; U.S. Government Sponsorhip under NASA Contract NAS7-918   *
X; is acknowledged.                                          *
X;                                                           *
X;***********************************************************/
X
X;;----------------------------------------------------------------------;;
X;;                                                                      ;;
X;;                        M A T R I C E S . S C M                       ;;
X;;                                                                      ;;
X;;----------------------------------------------------------------------;;
X;;; Section 3.4,  page 242 ff of Abelson & Sussman
X;;; Brian Beckman, 24 July 1989
X;;;                 5 Aug  1989 -- polishing
X;;----------------------------------------------------------------------;;
X
X;;; CONTENTS
X
X;;; (accum op init vec)
X;;; (flatten stream)
X;;; (filter-stream predicate? stream)
X;;; (accum-n op init streams)
X;;; (%m m)
X;;; (*vv v w)
X;;; (*mv m v)
X;;; (*vm v m)
X;;; (*mn m n)
X;;; (*vmv v m)
X;;; (@vv v w)
X;;; (*vs v s)
X;;; (*ms m s)
X;;; (skip-elt stream k)
X;;; (cofactor m i j)
X;;; (alt vec)
X;;; (det m)
X;;; (inverse m)
X;;; (+vv v w)
X;;; (-vw v w)
X;;; (+mm m n)
X;;; (-mn m n)
X;;; (zero dim)
X;;; (vconst c dim)
X;;; (e k dim)
X;;; (ident dim)
X;;; seed
X;;; (vrand max-elt dim)
X;;; (mrand max-elt dim)
X;;; (vclean v epsilon)
X;;; (mclean m epsilon)
X;;; (printm m)
X
X;;; For small matrices, we implement streams as ordinary lists.  For
X;;; large matrices, use kstreams.  Some kstream operators are macros,
X;;; and cannot be apply-ed or map-ped (e.g. kons), so some matrix
X;;; operators in this file have no analogous forms in kstream represen-
X;;; tation.
X
X;;----------------------------------------------------------------------;;
X;;                    s t r e a m   o p e r a t o r s                   ;;
X;;----------------------------------------------------------------------;;
X(define (accum op init vec)
X  (cond
X   (  (null? vec)  init  )
X   (  t  (op (car vec) (accum op init (cdr vec)))  )))
X
X(define (flatten stream)
X  (accum append '() stream))
X
X(define (filter-stream predicate? stream)
X  (cond
X   (  (null? stream)  '()  )
X   (  (predicate? (car stream))
X        (cons
X         (car stream)
X         (filter-stream predicate? (cdr stream)))  )
X   (  t  (filter-stream predicate? (cdr stream))  )))
X
X(define (accum-n op init streams)    ;; CROSS-SECTION ACROSS
X  (cond                              ;; A STREAM OF STREAMS
X   (  (null? (car streams))  '()  )
X   (  t  (cons
X          (accum   op init (map car streams))
X          (accum-n op init (map cdr streams)))  )))
X
X;;; page 252, Ex 3.40
X
X;;; an arbitrary-dimensional matrix library.
X;;; There is no error checking in this first draft.
X
X;;----------------------------------------------------------------------;;
X;;                           t r a n s p o s e                          ;;
X;;----------------------------------------------------------------------;;
X(define (%m m) (accum-n cons '() m))
X
X;;----------------------------------------------------------------------;;
X;;                      i n n e r   p r o d u c t s                     ;;
X;;----------------------------------------------------------------------;;
X(define (*vv v w) (accum + 0 (accum-n * 1 (list v w))))
X
X(define (*mv m v) (map (lambda (row) (*vv row      v))     m))
X
X(define (*vm v m) (map (lambda (row) (*vv row      v)) (%m m)))
X
X(define (*mn m n) (map (lambda (vec) (*mv (%m n) vec))     m))
X
X(define (*mn-f m n)
X  (let (( n (%m n) ))
X    (map (lambda (vec) (*mv n vec)) m)))
X;;; NO NOTICEABLE PERFORMANCE IMPROVEMENT
X
X(define (*vmv v m) (*vv (*vm v m) v))
X
X;;----------------------------------------------------------------------;;
X;;                       o u t e r   p r o d u c t                      ;;
X;;----------------------------------------------------------------------;;
X(define (@vv v w) (*mn (map list v) (list w)))
X
X;;----------------------------------------------------------------------;;
X;;                             s c a l i n g                            ;;
X;;----------------------------------------------------------------------;;
X(define (*vs v s) (map (lambda (x) (*   x s)) v))
X
X(define (*ms m s) (map (lambda (v) (*vs v s)) m))
X
X;;----------------------------------------------------------------------;;
X;;        determinant by cofactor analysis (naturally recursive)        ;;
X;;----------------------------------------------------------------------;;
X;;; This procedure is unpleasantly slow by dim = 6.
X
X(define (skip-elt stream k)
X  (define (iter s count)
X    (cond
X     ( (null? s)  '() )
X     ( (= k count) (cdr s) )
X     ( else (cons (car s) (iter (cdr s) (1+ count))))))
X  (iter stream 1))
X
X(define (cofactor m i j)
X  (define (iter k m)
X    (cond
X     ( (null? m)  '() )
X     ( (= k i)    (iter (1+ k) (cdr m)) )
X     ( else       (cons (skip-elt (car m) j)
X                        (iter (1+ k) (cdr m))) )))
X  (iter 1 m))
X
X(define (sign i)
X    (if (= 0 (remainder i 2)) 1 -1))
X
X(define (det m)  
X  (define (iter s j)
X    (cond
X     ( (null? s)  0 )
X     ( else  (+ (* (car s) (det (cofactor m 1 j)) (sign (1+ j)))
X                (iter (cdr s) (1+ j))) )))
X  (cond
X   ( (null? m)  1 )
X   ( else (iter (car m) 1) )))
X
X;;----------------------------------------------------------------------;;
X;;                             i n v e r s e                            ;;
X;;----------------------------------------------------------------------;;
X(define (inverse m)
X  (let (( d (det m) ))
X    (define (do-row i j r)
X      (cond
X       ( (null? r)  '() )
X       ( else (cons (/ (det (cofactor m i j)) (* (sign (+ i j)) d))
X                    (do-row i (1+ j) (cdr r))) )))
X    (define (iter i block)
X      (cond
X       ( (null? block) '() )
X       ( else (cons (do-row i 1 (car block))
X                    (iter (1+ i) (cdr block))) )))
X    (%m (iter 1 m))))
X
X;;----------------------------------------------------------------------;;
X;;               v e c t o r   a d d   &   s u b t r a c t              ;;
X;;----------------------------------------------------------------------;;
X(define (+vv v w) (accum-n + 0 (list v w)))
X
X(define (-vw v w) (accum-n - 0 (list v w)))
X
X;;----------------------------------------------------------------------;;
X;;               m a t r i x   a d d   &   s u b t r a c t              ;;
X;;----------------------------------------------------------------------;;
X(define (+mm m n) (accum-n +vv (zero (length (car m))) (list m n)))
X
X(define (-mn m n) (accum-n -vw (zero (length (car m))) (list m n)))
X
X;;----------------------------------------------------------------------;;
X;;                        z e r o   v e c t o r s                       ;;
X;;----------------------------------------------------------------------;;
X(define (zero dim)
X  (define (iter i)
X    (cond
X     ( (= i dim)  '() )
X     ( else  (cons 0 (iter (1+ i))) )))
X  (iter 0))
X
X;;----------------------------------------------------------------------;;
X;;                    c o n s t a n t   v e c t o r                     ;;
X;;----------------------------------------------------------------------;;
X(define (vconst c dim)
X  (define (iter i)
X    (cond
X     ( (= i dim)  '() )
X     ( else  (cons c (iter (1+ i))) )))
X  (iter 0))
X
X;;----------------------------------------------------------------------;;
X;;                       b a s i s   v e c t o r s                      ;;
X;;----------------------------------------------------------------------;;
X(define (e k dim)
X  (define (iter i)
X    (cond
X     ( (= i dim)  '() )
X     ( (= i k)  (cons 1 (iter (1+ i))) )
X     ( else     (cons 0 (iter (1+ i))) )))
X  (iter 0))
X
X;;----------------------------------------------------------------------;;
X;;                     i d e n t i t y   m a t r i x                    ;;
X;;----------------------------------------------------------------------;;
X(define (ident dim)
X  (define (iter i)
X    (cond
X     ( (= i dim)  '() )
X     ( else  (cons (e i dim) (iter (1+ i))) )))
X  (iter 0))
X
X;;----------------------------------------------------------------------;;
X;;                       r a n d o m   v e c t o r                      ;;
X;;----------------------------------------------------------------------;;
X(define seed
X  (letrec (( iter (lambda (i max)
X                    (cond
X                     ( (>= i max)  (random 256) )
X                     ( else  (random 256) (iter (1+ i) max) ))) ))
X    (delay (iter 0 (remainder (time) 256)))))
X
X(define (vrand max dim)
X  (define (iter i)
X    (cond
X     ( (>= i dim)  '() )
X     ( else  (cons (random max) (iter (1+ i))) )))
X  (force seed)
X  (iter 0))
X
X;;----------------------------------------------------------------------;;
X;;                       r a n d o m   m a t r i x                      ;;
X;;----------------------------------------------------------------------;;
X(define (mrand max dim)
X  (define (iter i)
X    (cond
X     ( (>= i dim)  '() )
X     ( else  (cons (vrand max dim) (iter (1+ i))) )))
X  (iter 0))
X
X;;----------------------------------------------------------------------;;
X;;            f l o a t i n g   p o i n t   c l e a n u p               ;;
X;;----------------------------------------------------------------------;;
X(define (vclean v epsilon)
X  (map (lambda (x) (if (< (abs x) epsilon) 0 x)) v))
X
X(define (mclean m epsilon)
X  (map (lambda (v) (vclean v epsilon)) m))
X
X;;----------------------------------------------------------------------;;
X;;                     p r i n t   r o u t i n e                        ;;
X;;----------------------------------------------------------------------;;
X(define (printm m)
X  (display "(")
X  (letrec (( iter
X             (lambda (y)
X               (cond
X                ( (null? y) (display ")") (newline) )
X                ( else  (display (car y)) (newline) 
X                        (display " ") (iter (cdr y)) ))) ))
X    (iter m)))
END_OF_FILE
if test 10072 -ne `wc -c <'methods/Examples/matrices/Matrices.scm'`; then
    echo shar: \"'methods/Examples/matrices/Matrices.scm'\" unpacked with wrong size!
fi
# end of 'methods/Examples/matrices/Matrices.scm'
fi
if test -f 'methods/Examples/utils/kstream.scm' -a "${1}" != "-c" ; then 
  echo shar: Will not clobber existing file \"'methods/Examples/utils/kstream.scm'\"
else
echo shar: Extracting \"'methods/Examples/utils/kstream.scm'\" \(9861 characters\)
sed "s/^X//" >'methods/Examples/utils/kstream.scm' <<'END_OF_FILE'
X; %
X; %    K S t r e a m s
X; %
X; % As in Abelson & Sussman, Section 3.4.3, page 259 ff.
X; %
X; Kstreams are an efficient implementation of long, or even infinite,
X; streams, using {\tt delay} and {\tt force}.  These Scheme primitives
X; provide delayed evaluation, which is necessary for decent
X; efficiency.  Without them, long streams are obscenely inefficient.
X;
X; I named these optimized streams kstreams just so I could write {\tt
X; kons} for the kstream konstructor.  The names of other kstream
X; objects all begin with ``k''.  The convention that the name of every
X; kstream object begin with ``k'' prevents name conflicts with Scheme
X; primitives such as {\tt cons} and {\tt map}.  This naming convention
X; also allows kstreams to coexist with streams implemented as ordinary
X; lists -- {\em Lstreams}. Streams so implemented are appropriate for
X; short streams, such as those representing vectors and matrices (see
X; page 252 of A&S and ``Lstream.scm'').  For short lists, the
X; function-call overhead inherent in {\tt delay} and {\tt force} is a
X; little too costly.
X;
X; The following operators are defined for KStreams:
X;
X; essential operators:
X;
X; (kons <head> <tail>)
X; (kar <ks>)
X; (kdr <ks>)
X; (kth <k> <ks>)
X; (kempty? <ks>)
X; knil % cute, huh?
X; (kappend <ks1> <ks2>)
X; (kaccum <combiner> <initial-value> <ks>)
X; (kflatten <ks>)
X; (kmap <proc> <ks>)
X; (kflatmap <proc> <ks>)
X; (kfilter <predicate?> <ks>)
X; (kfor-each <proc> <ks>)
X; (kprint <ks>)
X; (ks->list <ks>)
X; (klist->ks <list>)
X; (kstream <elt1> <elt2> ... <eltn>)
X; (kaccum-n <op> <initial-value> <ks-of-ks's>)
X; (kollect <result> ((<v1> <set1>) ... (<vn> <setn>)) <restriction>)
X;
X; application and example operators:
X;
X; (enumerate-interval <n1> <n2>)
X; (kleft-accum <combiner> <initial-value> <ks>)
X; (ks-of-pairs <max-number>)
X; (k*vv <vector1> <vector2>)
X; (k*mv <matrix> <vector>)
X; (k%m  <matrix>)
X; (k*mn <matrix1> <matrix2>)
X; (define (ks-of-pairs n)
X; (prime-sum-pairs n)
X; (kremove item ks)
X; (kpermutations s)
X; (integers-from n)
X; integers
X; no-sevens
X; (fibgen a b)
X; fibs
X; (sieve <ks>)
X; primes-sieve
X; primes
X; (prime? <n>)
X; (kadd <k1> <k2>)
X; (kscale <scalar> <ks>)
X
X
X(define knil ())
X
X(macro kons (lambda (l)
X              `(cons ,(cadr l) (delay ,(caddr l)))))
X;
X; The definition of {\tt enumerate-interval} is needed here to help
X; test the implementation of kons.
X;
X
X(define (enumerate-interval lo hi)
X  (if (> lo hi)
X      knil
X      (kons lo (enumerate-interval (1+ lo) hi))))
X
X(define (kar ks) (car ks))
X
X(define (kdr ks) (force (cdr ks)))
X
X(define (kth k ks)
X  (if (= k 0)
X      (kar ks)
X      (kth (-1+ k) (kdr ks))))
X
X(define kempty? null?)
X
X; The following variables are abstract operators on streams,
X; independent of the representation above.
X
X(define (kappend ks1 ks2)
X  (cond
X   (  (kempty? ks1)  ks2  )
X   (  t  (kons (kar ks1) (kappend (kdr ks1) ks2)))))
X
X(define (kaccum combiner initial-value ks)
X  (cond
X   (  (kempty? ks)  initial-value  )
X   (  t  (combiner
X          (kar ks)
X          (kaccum combiner initial-value (kdr ks)))  )))
X
X(define (kflatten ks)
X  (kaccum kappend knil ks))
X
X;
X; The following test application evaluates a polynomial, represented
X; by the kstream {\em coeff-stream} of its coefficients, at the
X; supplied point {\em x}.
X;
X
X(define (horner-eval x coeff-stream)
X  (define (add-term coeff higher-terms)
X    (+ coeff (* x higher-terms)))
X  (kaccum add-term 0 coeff-stream))
X
X(define (kfilter predicate? ks)
X  (cond
X   (  (kempty? ks)  knil  )
X   (  (predicate? (kar ks))
X        (kons
X         (kar ks)
X         (kfilter predicate? (kdr ks)))  )
X   (  t  (kfilter predicate? (kdr ks))  )))
X
X(define (kmap proc ks)
X  (cond
X   (  (kempty? ks)  knil  )
X   (  t  (kons
X          (proc (kar ks))
X          (kmap proc (kdr ks)))  )))
X
X(define (kflatmap f s) (kflatten (kmap f s)))
X
X(define (kfor-each proc ks)
X  (cond
X   (  (kempty? ks)  'done  )
X   (  t  (begin
X          (proc (kar ks))
X          (kfor-each proc (kdr ks)))  )))
X
X(define (probably-kstream? ks)
X  (cond
X   (  (kempty? ks)           t  )
X   (  (atom?   ks)           #f )
X   (  (procedure? (cdr ks))  t  )
X   (  t                      #f )))
X
X(define kprint
X  (letrec
X   ((foo
X     (lambda (ks)
X       (cond ((kempty? ks) (begin (display "]") (newline) t))
X             (t
X              (begin (let (  (h (kar ks))  )
X                       (if (probably-kstream? h) (kprint h)
X                                                 (display h)))
X                     (if (not (kempty? (kdr ks))) (display " "))
X                     (foo (kdr ks))))))))
X    (lambda (ks) (display "[") (foo ks))))
X
X;;; page 251, Ex. 3.38
X
X(define (kleft-accum combiner initial-value ks)
X  (cond
X   (  (kempty? ks)  initial-value  )
X   (  t  (kleft-accum combiner
X                      (combiner initial-value (kar ks))
X                      (kdr ks))  )))
X;;; end of exercise
X
X(define (ks->list ks) (kaccum cons () ks))
X
X(define (klist->ks list)
X  (cond
X   (  (null? list)  knil  )
X   (  t  (kons (car list) (klist->ks (cdr list)))  )))
X
X(define kstream
X  (lambda args
X    (cond
X     (  (null? args)  knil  )
X     (  t  (kons (car args) (klist->ks (cdr args)))  ))))
X
X;;; page 252, Ex. 3.39
X
X(define (kaccum-n op init ks-of-kss)
X  (cond
X   (  (kempty? (kar ks-of-kss))  knil  )
X   (  t  (kons
X          (kaccum   op init (kmap kar ks-of-kss))
X          (kaccum-n op init (kmap kdr ks-of-kss)))  )))
X
X;
X; The syntax extension library is needed at this point (see Dybvig).
X;
X; (load "top 20:[scheme]:contributed:extendsyntax:extendsyntax.sch")
X;
X; This library is necessary, but not sufficient for the definition of
X; {\tt kollect}.  {\tt Kollect} is syntactic sugar for nested {\tt
X; kflatten} and {\tt kmap} operators.  The innermost operator is {\tt
X; kmap}, and the outer operators are {\tt kflatmap}s.  {\tt Kflatmap},
X; of course, is simply a composition of {\tt kflatten} and {\tt kmap}.
X; Dybvig's {\tt extend-syntax} is not quite up to the task of creating
X; {\tt kollect}, since its ellipsis notation allows one to repeat an
X; expansion for all present arguments, but not to give special
X; treatment to the innermost expression of a sequence of expressions.
X; Either a new kstream operator, say {\tt kmap-or-kflatmap}, that
X; tests the contents of its stream argument, and only flattens if the
X; stream contains streams, or a new syntactic sugar is needed.
X;% @@ This is yet another thing to finish.
X;
X
X;% page 252, Ex 3.40
X
X;
X; The following application operators furnish an arbitrary-dimensional
X; matrix library.  Amazingly, they are all one-liners! The present
X; kstream version of this library is appropriate for very large
X; matrices, with dimension over 20 or so. For smaller matrices, the
X; library implemented in terms of ordinary lstreams -- streams as
X; lists -- is appropriate.
X;
X
X(define (k*vv v w) (kaccum + 0 (kaccum-n * 1 (kons v (kons w knil)))))
X
X(define (k*mv m v) (kmap (lambda (row) (k*vv row v)) m))
X
X; The following definition of {\tt k%m}, the transpose operator, doesn't
X; work because {\tt kons} is not a procedure, but a syntactic form.
X; However, it does express the intention.
X;
X; (define (k%m m) (kaccum-n kons () m))  ;; TRANSPOSE
X;
X; The correct definition of {\tt k%m} follows as a macro: @@ TO DO
X;
X
X(define (k*mm m n) (let ((%n (k%m n))) (kmap (lambda (v) (k*mv %n v)) m)))
X
X; Here are the examples from page 253, leading to the definition of
X; {\tt kollect}.  These examples are useful for testing, and that is why
X; they are defined at this point.
X
X(define (ks-of-pairs n)
X  (kflatten (kmap (lambda (i)
X                    (kmap (lambda (j) (list i j))
X                          (enumerate-interval 1 (-1+ i))))
X                  (enumerate-interval 1 n))))
X
X(load "scheme code:*.scm:primes.scm")
X
X(define (prime-sum-pairs n)
X  (kmap (lambda (pair)
X          (list (car pair)
X                (cadr pair)
X                (+ (car pair) (cadr pair))))
X        (kfilter (lambda (pair)
X                   (prime? (+ (car pair) (cadr pair))))
X                 (ks-of-pairs n))))
X
X(define (kremove item ks)
X  (kfilter (lambda (x) (not (equal? x item))) ks))
X
X(define (kpermutations s)
X  (cond
X   (  (kempty? s)  (kons () knil)  )
X   (  t  (kflatmap
X          (lambda (x)
X            (kmap
X             (lambda (p)
X               (kons x p))
X             (kpermutations (kremove x s))))
X          s)  )))
X
X;% Infinitely long streams.
X
X(define (integers-from n)
X  (kons n (integers-from (1+ n))))
X
X(define integers (integers-from 1))
X
X(define no-sevens
X  (kfilter (lambda (x) (not (divides? x 7)))
X           integers))
X
X(kth 100 no-sevens)
X
X(define (fibgen a b)
X  (kons a (fibgen b (+ a b))))
X
X(define fibs (fibgen 0 1))
X
X(define (sieve ks)
X  (kons
X   (kar ks)
X   (sieve
X    (kfilter
X     (lambda (x) (not (divides? (kar ks) x)))
X     (kdr ks)))))
X
X(define kprimes-sieve (sieve (integers-from 2)))
X
X(define kprimes
X  (kons 2 (kfilter kprime? (integers-from 3))))
X
X(define (kprime? n)
X  (define (iter ps)
X    (cond
X     (  (> (square (kar ps)) n)  t  )
X     (  (divides? (kar ps) n)  ()  )
X     (  else  (iter (kdr ps))  )))
X  (iter kprimes))
X
X(define (kfactors n)
X  (define ps kprimes)
X  (define (next factor)
X    (set! ps (kdr ps))
X    (kar ps))
X  (define squirt (sqrt n))
X  (define (iter factor n)
X    (cond
X     (  (< n 2)              ()  )
X     (  (> factor squirt)    (list n)  )
X     (  (divides? factor n)  (cons factor (iter factor (/ n factor)))  )
X     (  else                 (iter (next factor) n)  )))
X  (iter 2 n))
X
X(define (kprime-factors n)
X  (powers-list (kfactors n)))
X
X;% This baby is a hog for memory, so be sure to (define primes 0) (gc)
X;% when you're done.
X
X(define (kadd k1 k2)
X  (cond
X   (  (kempty? k1)  k2  )
X   (  (kempty? k2)  k1  )
X   (  t             (kons
X                     (+ (kar k1) (kar k2))
X                     (kadd (kdr k1) (kdr k2))))))
X
X(define (kscale scalar ks)
X  (kmap (lambda (x) (* x scalar)) ks))
END_OF_FILE
if test 9862 -ne `wc -c <'methods/Examples/utils/kstream.scm'`; then
    echo shar: \"'methods/Examples/utils/kstream.scm'\" unpacked with wrong size!
fi
# end of 'methods/Examples/utils/kstream.scm'
fi
echo shar: End of archive 2 \(of 2\).
cp /dev/null ark2isdone
MISSING=""
for I in 1 2 ; do
    if test ! -f ark${I}isdone ; then
	MISSING="${MISSING} ${I}"
    fi
done
if test "${MISSING}" = "" ; then
    echo You have unpacked both archives.
    rm -f ark[1-9]isdone
else
    echo You still need to unpack the following archives:
    echo "        " ${MISSING}
fi
##  End of shell archive.
exit 0
